from dj_rest_auth.registration.serializers import \
    RegisterSerializer as DefaultRegisterSerializer
from django.conf import settings
from rest_framework.permissions import AllowAny

from ..utils import import_callable

serializers = getattr(settings, 'REST_AUTH_REGISTER_SERIALIZERS', {})

RegisterSerializer = import_callable(serializers.get('REGISTER_SERIALIZER', DefaultRegisterSerializer))


def register_permission_classes():
    permission_classes = [AllowAny, ]
    for klass in getattr(settings, 'REST_AUTH_REGISTER_PERMISSION_CLASSES', tuple()):
        permission_classes.append(klass)
    return tuple(permission_classes)



from importlib import import_module


def import_callable(path_or_callable):
    if hasattr(path_or_callable, '__call__'):
        return path_or_callable
    else:
        assert isinstance(path_or_callable, str)
        package, attr = path_or_callable.rsplit('.', 1)
        return getattr(import_module(package), attr)


def default_create_token(token_model, user, serializer):
    token, _ = token_model.objects.get_or_create(user=user)
    return token


def jwt_encode(user):
    try:
        from rest_framework_simplejwt.serializers import TokenObtainPairSerializer
    except ImportError:
        raise ImportError("rest-framework-simplejwt needs to be installed")

    refresh = TokenObtainPairSerializer.get_token(user)
    return refresh.access_token, refresh


try:
    from rest_framework_simplejwt.authentication import JWTAuthentication

    class JWTCookieAuthentication(JWTAuthentication):
        """
        An authentication plugin that hopefully authenticates requests through a JSON web
        token provided in a request cookie (and through the header as normal, with a
        preference to the header).
        """
        def authenticate(self, request):
            from django.conf import settings
            cookie_name = getattr(settings, 'JWT_AUTH_COOKIE', None)
            header = self.get_header(request)
            if header is None:
                if cookie_name:
                    raw_token = request.COOKIES.get(cookie_name)
                else:
                    return None
            else:
                raw_token = self.get_raw_token(header)
                
            if raw_token is None:
                return None

            validated_token = self.get_validated_token(raw_token)
            return self.get_user(validated_token), validated_token

except ImportError:
    pass



from django.conf import settings
from rest_framework.authtoken.models import Token as DefaultTokenModel

# Register your models here.

TokenModel = getattr(settings, 'REST_AUTH_TOKEN_MODEL', DefaultTokenModel)



from dj_rest_auth.serializers import JWTSerializer as DefaultJWTSerializer
from dj_rest_auth.serializers import LoginSerializer as DefaultLoginSerializer
from dj_rest_auth.serializers import \
    PasswordChangeSerializer as DefaultPasswordChangeSerializer
from dj_rest_auth.serializers import \
    PasswordResetConfirmSerializer as DefaultPasswordResetConfirmSerializer
from dj_rest_auth.serializers import \
    PasswordResetSerializer as DefaultPasswordResetSerializer
from dj_rest_auth.serializers import TokenSerializer as DefaultTokenSerializer
from dj_rest_auth.serializers import \
    UserDetailsSerializer as DefaultUserDetailsSerializer
from django.conf import settings

from .utils import default_create_token, import_callable

create_token = import_callable(getattr(settings, 'REST_AUTH_TOKEN_CREATOR', default_create_token))

serializers = getattr(settings, 'REST_AUTH_SERIALIZERS', {})

TokenSerializer = import_callable(serializers.get('TOKEN_SERIALIZER', DefaultTokenSerializer))

JWTSerializer = import_callable(serializers.get('JWT_SERIALIZER', DefaultJWTSerializer))

UserDetailsSerializer = import_callable(serializers.get('USER_DETAILS_SERIALIZER', DefaultUserDetailsSerializer))

LoginSerializer = import_callable(serializers.get('LOGIN_SERIALIZER', DefaultLoginSerializer))

PasswordResetSerializer = import_callable(serializers.get(
    'PASSWORD_RESET_SERIALIZER', DefaultPasswordResetSerializer
))

PasswordResetConfirmSerializer = serializers.get(
    'PASSWORD_RESET_CONFIRM_SERIALIZER', DefaultPasswordResetConfirmSerializer
)

PasswordChangeSerializer = import_callable(serializers.get('PASSWORD_CHANGE_SERIALIZER', DefaultPasswordChangeSerializer))

JWT_AUTH_COOKIE = getattr(settings, 'JWT_AUTH_COOKIE', None)
